@@ -4,6 +4,7 @@
  */
 
 #include <assert.h>
+#include <at91_clk.h>
 #include <config.h>
 #include <drivers/clk.h>
 #include <drivers/clk_dt.h>
@@ -36,7 +37,7 @@ static TEE_Result get_freq_from_dt(void)
 
 	freq = clk_get_rate(clk);
 
-	return TEE_SUCCESS;
+	return at91_clk_register_cpu_opp(fdt, node, clk);
 }
 early_init_late(get_freq_from_dt);
 
