@@ -18,6 +18,8 @@
 /*
  * SYSCFG register offsets (base relative)
  */
+#define SYSCFG_SRAM3ERASER			U(0x10)
+#define SYSCFG_SRAM3KR				U(0x14)
 #define SYSCFG_IOCTRLSETR			U(0x18)
 #define SYSCFG_CMPCR				U(0x20)
 #define SYSCFG_CMPENSETR			U(0x24)
@@ -36,6 +38,17 @@
 #define SYSCFG_IOCTRLSETR_HSLVEN_SDMMC		BIT(3)
 #define SYSCFG_IOCTRLSETR_HSLVEN_SPI		BIT(4)
 
+/*
+ * SYSCFG_SRAM3ERASE Register
+ */
+#define SYSCFG_SRAM3KR_KEY1			U(0xCA)
+#define SYSCFG_SRAM3KR_KEY2			U(0x53)
+
+#define SYSCFG_SRAM3ERASER_SRAM3EO		BIT(1)
+#define SYSCFG_SRAM3ERASER_SRAM3ER		BIT(0)
+
+#define SYSCFG_SRAM3ERASE_TIMEOUT_US		U(1000)
+
 /*
  * SYSCFG_CMPCR Register
  */
@@ -46,7 +59,7 @@
 #define SYSCFG_CMPCR_RAPSRC			GENMASK_32(23, 20)
 #define SYSCFG_CMPCR_ANSRC_SHIFT		U(24)
 
-#define SYSCFG_CMPCR_READY_TIMEOUT_US		U(1000)
+#define SYSCFG_CMPCR_READY_TIMEOUT_US		U(10000)
 
 #define CMPENSETR_OFFSET			U(0x4)
 #define CMPENCLRR_OFFSET			U(0x8)
@@ -139,6 +152,9 @@ static TEE_Result stm32mp1_iocomp(void)
 
 	enable_io_compensation(SYSCFG_CMPCR);
 
+	/* Make sure the write above is visible */
+	dsb();
+
 	return TEE_SUCCESS;
 }
 
@@ -149,6 +165,9 @@ void stm32mp_set_vddsd_comp_state(enum stm32mp13_vddsd_comp_id id, bool enable)
 {
 	int cmpcr_offset = 0;
 
+	/* Make sure the previous operations are visible */
+	dsb();
+
 	switch (id) {
 	case SYSCFG_IO_COMP_IDX_SD1:
 		cmpcr_offset = SYSCFG_CMPSD1CR;
@@ -164,6 +183,9 @@ void stm32mp_set_vddsd_comp_state(enum stm32mp13_vddsd_comp_id id, bool enable)
 		enable_io_compensation(cmpcr_offset);
 	else
 		disable_io_compensation(cmpcr_offset);
+
+	/* Make sure the write above is visible */
+	dsb();
 }
 
 void stm32mp_set_hslv_state(enum stm32mp13_hslv_id id, bool enable)
@@ -171,6 +193,9 @@ void stm32mp_set_hslv_state(enum stm32mp13_hslv_id id, bool enable)
 	size_t hslvenxr_offset = 0;
 	uint32_t hlvs_value = 0;
 
+	/* Make sure the previous operations are visible */
+	dsb();
+
 	assert(id < SYSCFG_HSLV_COUNT);
 
 	if (enable)
@@ -206,11 +231,42 @@ void stm32mp_enable_fixed_vdd_hslv(void)
 #ifdef CFG_STM32MP15
 void stm32mp_enable_fixed_vdd_hslv(void)
 {
+	dsb();
+
 	io_write32(get_syscfg_base() + SYSCFG_IOCTRLSETR,
 		   SYSCFG_IOCTRLSETR_HSLVEN_TRACE |
 		   SYSCFG_IOCTRLSETR_HSLVEN_QUADSPI |
 		   SYSCFG_IOCTRLSETR_HSLVEN_ETH |
 		   SYSCFG_IOCTRLSETR_HSLVEN_SDMMC |
 		   SYSCFG_IOCTRLSETR_HSLVEN_SPI);
+
+	dsb();
 }
 #endif
+
+TEE_Result stm32mp_syscfg_erase_sram3(void)
+{
+	vaddr_t base = get_syscfg_base();
+	uint32_t value = 0;
+
+	if (!IS_ENABLED(CFG_STM32MP13))
+		return TEE_ERROR_NOT_SUPPORTED;
+
+	/* Unlock SYSCFG_SRAM3ERASER_SRAM3ER */
+	io_write32(base + SYSCFG_SRAM3KR, SYSCFG_SRAM3KR_KEY1);
+	io_write32(base + SYSCFG_SRAM3KR, SYSCFG_SRAM3KR_KEY2);
+
+	/* Request SRAM3 erase */
+	io_setbits32(base + SYSCFG_SRAM3ERASER, SYSCFG_SRAM3ERASER_SRAM3ER);
+
+	/* Lock SYSCFG_SRAM3ERASER_SRAM3ER */
+	io_write32(base + SYSCFG_SRAM3KR, 0);
+
+	/* Wait end of SRAM3 erase */
+	if (IO_READ32_POLL_TIMEOUT(base + SYSCFG_SRAM3ERASER, value,
+				   !(value & SYSCFG_SRAM3ERASER_SRAM3EO), 0,
+				   SYSCFG_SRAM3ERASE_TIMEOUT_US))
+		return TEE_ERROR_BUSY;
+
+	return TEE_SUCCESS;
+}
