@@ -235,6 +235,9 @@ static TEE_Result sam9x60_frac_pll_set_rate_chg(struct clk *hw,
 
 	ret = sam9x60_frac_pll_compute_mul_frac(frac, rate, parent_rate, true);
 	if (ret == TEE_SUCCESS) {
+		io_clrsetbits32(regmap + AT91_PMC_PLL_UPDT,
+				AT91_PMC_PLL_UPDT_ID_MASK, core->id);
+
 		io_write32(regmap + AT91_PMC_PLL_CTRL1,
 			   SHIFT_U32(frac->mul, core->layout->mul_shift) |
 			   SHIFT_U32(frac->frac, core->layout->frac_shift));
@@ -277,6 +280,9 @@ static TEE_Result sam9x60_div_pll_set_div(struct sam9x60_pll_core *core,
 	uint32_t enable_mask = enable ? core->layout->endiv_mask : 0;
 	uint32_t ena_val = enable ? BIT(core->layout->endiv_shift) : 0;
 
+	io_clrsetbits32(regmap + AT91_PMC_PLL_UPDT,
+			AT91_PMC_PLL_UPDT_ID_MASK, core->id);
+
 	io_clrsetbits32(regmap + AT91_PMC_PLL_CTRL0,
 			core->layout->div_mask | enable_mask,
 			SHIFT_U32(div, core->layout->div_shift) | ena_val);
@@ -349,7 +355,10 @@ static TEE_Result sam9x60_div_pll_set_rate(struct clk *hw,
 {
 	struct sam9x60_div *div = hw->priv;
 
-	div->div = UDIV_ROUND_NEAREST(parent_rate, rate) - 1;
+	if (parent_rate > rate)
+		div->div = UDIV_ROUND_NEAREST(parent_rate, rate) - 1;
+	else
+		div->div = 0;
 
 	return TEE_SUCCESS;
 }
@@ -364,7 +373,10 @@ static TEE_Result sam9x60_div_pll_set_rate_chg(struct clk *hw,
 	unsigned int val = 0;
 	unsigned int cdiv = 0;
 
-	div->div = UDIV_ROUND_NEAREST(parent_rate, rate) - 1;
+	if (parent_rate > rate)
+		div->div = UDIV_ROUND_NEAREST(parent_rate, rate) - 1;
+	else
+		div->div = 0;
 
 	io_clrsetbits32(regmap + AT91_PMC_PLL_UPDT,
 			AT91_PMC_PLL_UPDT_ID_MASK,
