@@ -1,6 +1,6 @@
 // SPDX-License-Identifier: BSD-2-Clause
 /*
- * Copyright (c) 2016, 2022 Linaro Limited
+ * Copyright (c) 2016-2025 Linaro Limited
  * Copyright (c) 2014, STMicroelectronics International N.V.
  * Copyright (c) 2022, Arm Limited and Contributors. All rights reserved.
  */
@@ -62,19 +62,7 @@ unsigned long default_nsec_shm_size __nex_bss;
 unsigned long default_nsec_shm_paddr __nex_bss;
 #endif
 
-#ifdef CFG_BOOT_MEM
 static struct memory_map static_memory_map __nex_bss;
-#else
-static struct tee_mmap_region static_mmap_regions[CFG_MMAP_REGIONS
-#if defined(CFG_CORE_ASLR) || defined(CFG_CORE_PHYS_RELOCATABLE)
-						+ 1
-#endif
-						+ 4] __nex_bss;
-static struct memory_map static_memory_map __nex_data = {
-	.map = static_mmap_regions,
-	.alloc_count = ARRAY_SIZE(static_mmap_regions),
-};
-#endif
 void (*memory_map_realloc_func)(struct memory_map *mem_map) __nex_bss;
 
 /* Offset of the first TEE RAM mapping from start of secure RAM */
@@ -404,43 +392,54 @@ static void carve_out_phys_mem(struct core_mmu_phys_mem **mem, size_t *nelems,
 	struct core_mmu_phys_mem *m = *mem;
 	size_t n = 0;
 
-	while (true) {
-		if (n >= *nelems) {
-			DMSG("No need to carve out %#" PRIxPA " size %#zx",
-			     pa, size);
-			return;
+	while (n < *nelems) {
+		if (!core_is_buffer_intersect(pa, size, m[n].addr, m[n].size)) {
+			n++;
+			continue;
 		}
-		if (core_is_buffer_inside(pa, size, m[n].addr, m[n].size))
-			break;
-		if (!core_is_buffer_outside(pa, size, m[n].addr, m[n].size))
-			panic();
-		n++;
-	}
 
-	if (pa == m[n].addr && size == m[n].size) {
-		/* Remove this entry */
-		(*nelems)--;
-		memmove(m + n, m + n + 1, sizeof(*m) * (*nelems - n));
-		m = nex_realloc(m, sizeof(*m) * *nelems);
-		if (!m)
-			panic();
-		*mem = m;
-	} else if (pa == m[n].addr) {
-		m[n].addr += size;
-		m[n].size -= size;
-	} else if ((pa + size) == (m[n].addr + m[n].size)) {
-		m[n].size -= size;
-	} else {
-		/* Need to split the memory entry */
-		m = nex_realloc(m, sizeof(*m) * (*nelems + 1));
-		if (!m)
-			panic();
-		*mem = m;
-		memmove(m + n + 1, m + n, sizeof(*m) * (*nelems - n));
-		(*nelems)++;
-		m[n].size = pa - m[n].addr;
-		m[n + 1].size -= size + m[n].size;
-		m[n + 1].addr = pa + size;
+		if (core_is_buffer_inside(m[n].addr, m[n].size, pa, size)) {
+			/* m[n] is completely covered by pa:size */
+			rem_array_elem(m, *nelems, sizeof(*m), n);
+			(*nelems)--;
+			m = nex_realloc(m, sizeof(*m) * *nelems);
+			if (!m)
+				panic();
+			*mem = m;
+			continue;
+		}
+
+		if (pa > m[n].addr &&
+		    pa + size - 1 < m[n].addr + m[n].size - 1) {
+			/*
+			 * pa:size is strictly inside m[n] range so split
+			 * m[n] entry.
+			 */
+			m = nex_realloc(m, sizeof(*m) * (*nelems + 1));
+			if (!m)
+				panic();
+			*mem = m;
+			(*nelems)++;
+			ins_array_elem(m, *nelems, sizeof(*m), n + 1, NULL);
+			m[n + 1].addr = pa + size;
+			m[n + 1].size = m[n].addr + m[n].size - pa - size;
+			m[n].size = pa - m[n].addr;
+			n++;
+		} else if (pa <= m[n].addr) {
+			/*
+			 * pa:size is overlapping (possibly partially) at the
+			 * beginning of m[n].
+			 */
+			m[n].size = m[n].addr + m[n].size - pa - size;
+			m[n].addr = pa + size;
+		} else {
+			/*
+			 * pa:size is overlapping (possibly partially) at
+			 * the end of m[n].
+			 */
+			m[n].size = pa - m[n].addr;
+		}
+		n++;
 	}
 }
 
@@ -524,6 +523,8 @@ void core_mmu_set_discovered_nsec_ddr(struct core_mmu_phys_mem *start,
 		case MEM_AREA_SHM_VASPACE:
 		case MEM_AREA_TS_VASPACE:
 		case MEM_AREA_PAGER_VASPACE:
+		case MEM_AREA_NEX_DYN_VASPACE:
+		case MEM_AREA_TEE_DYN_VASPACE:
 			break;
 		default:
 			check_phys_mem_is_outside(m, num_elems,
@@ -534,6 +535,11 @@ void core_mmu_set_discovered_nsec_ddr(struct core_mmu_phys_mem *start,
 	discovered_nsec_ddr_start = m;
 	discovered_nsec_ddr_nelems = num_elems;
 
+	DMSG("Non-secure RAM:");
+	for (n = 0; n < num_elems; n++)
+		DMSG("%zu: pa %#"PRIxPA"..%#"PRIxPA" sz %#"PRIxPASZ,
+		     n, m[n].addr, m[n].addr + m[n].size - 1, m[n].size);
+
 	if (!core_mmu_check_end_pa(m[num_elems - 1].addr,
 				   m[num_elems - 1].size))
 		panic();
@@ -681,6 +687,16 @@ static void verify_special_mem_areas(struct memory_map *mem_map,
 	 */
 	for (mem = start; mem < end; mem++) {
 		for (n = 0; n < mem_map->count; n++) {
+#ifdef TEE_SDP_TEST_MEM_BASE
+			/*
+			 * Ignore MEM_AREA_SEC_RAM_OVERALL since it covers
+			 * TEE_SDP_TEST_MEM too.
+			 */
+			if (mem->addr == TEE_SDP_TEST_MEM_BASE &&
+			    mem->size == TEE_SDP_TEST_MEM_SIZE &&
+			    mem_map->map[n].type == MEM_AREA_SEC_RAM_OVERALL)
+				continue;
+#endif
 			if (core_is_buffer_intersect(mem->addr, mem->size,
 						     mem_map->map[n].pa,
 						     mem_map->map[n].size)) {
@@ -828,6 +844,8 @@ uint32_t core_mmu_type_to_attr(enum teecore_memtypes t)
 	case MEM_AREA_TEE_RAM_RW:
 	case MEM_AREA_NEX_RAM_RO: /* This has to be r/w during init runtime */
 	case MEM_AREA_NEX_RAM_RW:
+	case MEM_AREA_NEX_DYN_VASPACE:
+	case MEM_AREA_TEE_DYN_VASPACE:
 	case MEM_AREA_TEE_ASAN:
 		return attr | TEE_MATTR_SECURE | TEE_MATTR_PRW | tagged;
 	case MEM_AREA_TEE_COHERENT:
@@ -1069,7 +1087,7 @@ static void collect_device_mem_ranges(struct memory_map *mem_map)
 		}
 
 		add_phys_mem(mem_map, name, MEM_AREA_IO_SEC,
-			     base, base + page_count * SMALL_PAGE_SIZE);
+			     base, page_count * SMALL_PAGE_SIZE);
 	}
 }
 
@@ -1161,14 +1179,8 @@ static void collect_mem_ranges(struct memory_map *mem_map)
 		ADD_PHYS_MEM(MEM_AREA_SEC_RAM_OVERALL, secure_only[n].paddr,
 			     secure_only[n].size);
 
-	if (IS_ENABLED(CFG_CORE_SANITIZE_KADDRESS) &&
-	    IS_ENABLED(CFG_WITH_PAGER)) {
-		/*
-		 * Asan ram is part of MEM_AREA_TEE_RAM_RW when pager is
-		 * disabled.
-		 */
+	if (IS_ENABLED(CFG_CORE_SANITIZE_KADDRESS))
 		ADD_PHYS_MEM(MEM_AREA_TEE_ASAN, ASAN_MAP_PA, ASAN_MAP_SZ);
-	}
 
 #undef ADD_PHYS_MEM
 
@@ -1190,6 +1202,14 @@ static void collect_mem_ranges(struct memory_map *mem_map)
 
 	add_va_space(mem_map, MEM_AREA_RES_VASPACE, CFG_RESERVED_VASPACE_SIZE);
 	add_va_space(mem_map, MEM_AREA_SHM_VASPACE, SHM_VASPACE_SIZE);
+	if (IS_ENABLED(CFG_DYN_CONFIG)) {
+		if (IS_ENABLED(CFG_NS_VIRTUALIZATION))
+			add_va_space(mem_map, MEM_AREA_NEX_DYN_VASPACE,
+				     ROUNDUP(CFG_NEX_DYN_VASPACE_SIZE,
+					     CORE_MMU_PGDIR_SIZE));
+		add_va_space(mem_map, MEM_AREA_TEE_DYN_VASPACE,
+			     CFG_TEE_DYN_VASPACE_SIZE);
+	}
 }
 
 static void assign_mem_granularity(struct memory_map *mem_map)
@@ -1231,8 +1251,9 @@ static bool assign_mem_va_dir(vaddr_t tee_ram_va, struct memory_map *mem_map,
 			      bool tee_ram_at_top)
 {
 	struct tee_mmap_region *map = NULL;
-	vaddr_t va = 0;
+	bool va_is_nex_shared = false;
 	bool va_is_secure = true;
+	vaddr_t va = 0;
 	size_t n = 0;
 
 	/*
@@ -1263,7 +1284,7 @@ static bool assign_mem_va_dir(vaddr_t tee_ram_va, struct memory_map *mem_map,
 			map->va = va;
 			if (ADD_OVERFLOW(va, map->size, &va))
 				return false;
-			if (va >= BIT64(core_mmu_get_va_width()))
+			if (!core_mmu_va_is_valid(va))
 				return false;
 		}
 	}
@@ -1284,6 +1305,10 @@ static bool assign_mem_va_dir(vaddr_t tee_ram_va, struct memory_map *mem_map,
 			    va_is_secure != map_is_secure(map)) {
 				va_is_secure = !va_is_secure;
 				va = ROUNDDOWN(va, CORE_MMU_PGDIR_SIZE);
+			} else if (va_is_nex_shared !=
+				   core_mmu_type_is_nex_shared(map->type)) {
+				va_is_nex_shared = !va_is_nex_shared;
+				va = ROUNDDOWN(va, CORE_MMU_PGDIR_SIZE);
 			}
 
 			if (SUB_OVERFLOW(va, map->size, &va))
@@ -1318,6 +1343,12 @@ static bool assign_mem_va_dir(vaddr_t tee_ram_va, struct memory_map *mem_map,
 				if (ROUNDUP_OVERFLOW(va, CORE_MMU_PGDIR_SIZE,
 						     &va))
 					return false;
+			} else if (va_is_nex_shared !=
+				   core_mmu_type_is_nex_shared(map->type)) {
+				va_is_nex_shared = !va_is_nex_shared;
+				if (ROUNDUP_OVERFLOW(va, CORE_MMU_PGDIR_SIZE,
+						     &va))
+					return false;
 			}
 
 			if (ROUNDUP2_OVERFLOW(va, map->region_size, &va))
@@ -1338,7 +1369,7 @@ static bool assign_mem_va_dir(vaddr_t tee_ram_va, struct memory_map *mem_map,
 			map->va = va;
 			if (ADD_OVERFLOW(va, map->size, &va))
 				return false;
-			if (va >= BIT64(core_mmu_get_va_width()))
+			if (!core_mmu_va_is_valid(va))
 				return false;
 		}
 	}
@@ -1398,6 +1429,15 @@ static int cmp_init_mem_map(const void *a, const void *b)
 	if (!rc && !IS_ENABLED(CFG_WITH_LPAE))
 		rc = CMP_TRILEAN(map_is_secure(mm_a), map_is_secure(mm_b));
 
+	/*
+	 * Nexus mappings shared between partitions should not be mixed
+	 * with other mappings in the same translation table. Hence sort
+	 * nexus shared mappings from other mappings.
+	 */
+	if (!rc)
+		rc = CMP_TRILEAN(core_mmu_type_is_nex_shared(mm_a->type),
+				 core_mmu_type_is_nex_shared(mm_b->type));
+
 	return rc;
 }
 
@@ -1461,17 +1501,11 @@ static struct memory_map *init_mem_map(struct memory_map *mem_map,
 		add_pager_vaspace(mem_map);
 
 	if (IS_ENABLED(CFG_CORE_ASLR) && seed) {
-		vaddr_t base_addr = start_addr + seed;
-		const unsigned int va_width = core_mmu_get_va_width();
-		const vaddr_t va_mask = GENMASK_64(va_width - 1,
-						   SMALL_PAGE_SHIFT);
-		vaddr_t ba = base_addr;
+		vaddr_t ba = 0;
 		size_t n = 0;
 
 		for (n = 0; n < 3; n++) {
-			if (n)
-				ba = base_addr ^ BIT64(va_width - n);
-			ba &= va_mask;
+			ba = arch_aslr_base_addr(start_addr, seed, n);
 			if (assign_mem_va(ba, mem_map) &&
 			    mem_map_add_id_map(mem_map, id_map_start,
 					       id_map_end)) {
@@ -1540,6 +1574,8 @@ static void check_mem_map(struct memory_map *mem_map)
 		case MEM_AREA_RES_VASPACE:
 		case MEM_AREA_SHM_VASPACE:
 		case MEM_AREA_PAGER_VASPACE:
+		case MEM_AREA_NEX_DYN_VASPACE:
+		case MEM_AREA_TEE_DYN_VASPACE:
 			break;
 		default:
 			EMSG("Uhandled memtype %d", m->type);
@@ -1568,7 +1604,11 @@ void __weak core_init_mmu_map(unsigned long seed, struct core_mmu_config *cfg)
 	vaddr_t start = ROUNDDOWN((vaddr_t)__vcore_nex_rw_start,
 				  SMALL_PAGE_SIZE);
 #endif
+#ifdef CFG_DYN_CONFIG
+	vaddr_t len = ROUNDUP(VCORE_FREE_END_PA, SMALL_PAGE_SIZE) - start;
+#else
 	vaddr_t len = ROUNDUP((vaddr_t)__nozi_end, SMALL_PAGE_SIZE) - start;
+#endif
 	struct tee_mmap_region tmp_mmap_region = { };
 	struct memory_map mem_map = { };
 	unsigned long offs = 0;
@@ -1579,15 +1619,11 @@ void __weak core_init_mmu_map(unsigned long seed, struct core_mmu_config *cfg)
 
 	check_sec_nsec_mem_config();
 
-	if (IS_ENABLED(CFG_BOOT_MEM)) {
-		mem_map.alloc_count = CFG_MMAP_REGIONS;
-		mem_map.map = boot_mem_alloc_tmp(mem_map.alloc_count *
-							sizeof(*mem_map.map),
-						 alignof(*mem_map.map));
-		memory_map_realloc_func = boot_mem_realloc_memory_map;
-	} else {
-		mem_map = static_memory_map;
-	}
+	mem_map.alloc_count = CFG_MMAP_REGIONS;
+	mem_map.map = boot_mem_alloc_tmp(mem_map.alloc_count *
+						sizeof(*mem_map.map),
+					 alignof(*mem_map.map));
+	memory_map_realloc_func = boot_mem_realloc_memory_map;
 
 	static_memory_map = (struct memory_map){
 		.map = &tmp_mmap_region,
@@ -1620,20 +1656,17 @@ void __weak core_init_mmu_map(unsigned long seed, struct core_mmu_config *cfg)
 
 void core_mmu_save_mem_map(void)
 {
-	if (IS_ENABLED(CFG_BOOT_MEM)) {
-		size_t alloc_count = static_memory_map.count + 5;
-		size_t elem_sz = sizeof(*static_memory_map.map);
-		void *p = NULL;
+	size_t alloc_count = static_memory_map.count + 5;
+	size_t elem_sz = sizeof(*static_memory_map.map);
+	void *p = NULL;
 
-		p = nex_calloc(alloc_count, elem_sz);
-		if (!p)
-			panic();
-		memcpy(p, static_memory_map.map,
-		       static_memory_map.count * elem_sz);
-		static_memory_map.map = p;
-		static_memory_map.alloc_count = alloc_count;
-		memory_map_realloc_func = heap_realloc_memory_map;
-	}
+	p = nex_calloc(alloc_count, elem_sz);
+	if (!p)
+		panic();
+	memcpy(p, static_memory_map.map, static_memory_map.count * elem_sz);
+	static_memory_map.map = p;
+	static_memory_map.alloc_count = alloc_count;
+	memory_map_realloc_func = heap_realloc_memory_map;
 }
 
 bool core_mmu_mattr_is_ok(uint32_t mattr)
@@ -1923,16 +1956,19 @@ static bool can_map_at_level(paddr_t paddr, vaddr_t vaddr,
 
 void core_mmu_map_region(struct mmu_partition *prtn, struct tee_mmap_region *mm)
 {
-	struct core_mmu_table_info tbl_info;
-	unsigned int idx;
+	struct core_mmu_table_info tbl_info = { };
+	unsigned int idx = 0;
 	vaddr_t vaddr = mm->va;
 	paddr_t paddr = mm->pa;
 	ssize_t size_left = mm->size;
-	unsigned int level;
-	bool table_found;
-	uint32_t old_attr;
+	uint32_t attr = mm->attr;
+	unsigned int level = 0;
+	bool table_found = false;
+	uint32_t old_attr = 0;
 
 	assert(!((vaddr | paddr) & SMALL_PAGE_MASK));
+	if (!paddr)
+		attr = 0;
 
 	while (size_left > 0) {
 		level = CORE_MMU_BASE_TABLE_LEVEL;
@@ -1971,8 +2007,15 @@ void core_mmu_map_region(struct mmu_partition *prtn, struct tee_mmap_region *mm)
 			if (old_attr)
 				panic("Page is already mapped");
 
-			core_mmu_set_entry(&tbl_info, idx, paddr, mm->attr);
-			paddr += block_size;
+			core_mmu_set_entry(&tbl_info, idx, paddr, attr);
+			/*
+			 * Dynamic vaspace regions don't have a physical
+			 * address initially but we need to allocate and
+			 * initialize the translation tables now for later
+			 * updates to work properly.
+			 */
+			if (paddr)
+				paddr += block_size;
 			vaddr += block_size;
 			size_left -= block_size;
 
@@ -2131,23 +2174,74 @@ static bool mem_range_is_in_vcore_free(vaddr_t vstart, size_t num_pages)
 				     VCORE_FREE_PA, VCORE_FREE_SZ);
 }
 
+static void maybe_remove_from_mem_map(vaddr_t vstart, size_t num_pages)
+{
+	struct memory_map *mem_map = NULL;
+	struct tee_mmap_region *mm = NULL;
+	size_t idx = 0;
+	vaddr_t va = 0;
+
+	mm = find_map_by_va((void *)vstart);
+	if (!mm || !va_is_in_map(mm, vstart + num_pages * SMALL_PAGE_SIZE - 1))
+		panic("VA does not belong to any known mm region");
+
+	if (core_mmu_is_dynamic_vaspace(mm))
+		return;
+
+	if (!mem_range_is_in_vcore_free(vstart, num_pages))
+		panic("Trying to unmap static region");
+
+	/*
+	 * We're going to remove a memory from the VCORE_FREE memory range.
+	 * Depending where the range is we may need to remove the matching
+	 * mm, peal of a bit from the start or end of the mm, or split it
+	 * into two with a whole in the middle.
+	 */
+
+	va = ROUNDDOWN(vstart, SMALL_PAGE_SIZE);
+	assert(mm->region_size == SMALL_PAGE_SIZE);
+
+	if (va == mm->va && mm->size == num_pages * SMALL_PAGE_SIZE) {
+		mem_map = get_memory_map();
+		idx = mm - mem_map->map;
+		assert(idx < mem_map->count);
+
+		rem_array_elem(mem_map->map, mem_map->count,
+			       sizeof(*mem_map->map), idx);
+		mem_map->count--;
+	} else if (va == mm->va) {
+		mm->va += num_pages * SMALL_PAGE_SIZE;
+		mm->pa += num_pages * SMALL_PAGE_SIZE;
+		mm->size -= num_pages * SMALL_PAGE_SIZE;
+	} else if (va + num_pages * SMALL_PAGE_SIZE == mm->va + mm->size) {
+		mm->size -= num_pages * SMALL_PAGE_SIZE;
+	} else {
+		struct tee_mmap_region m = *mm;
+
+		mem_map = get_memory_map();
+		idx = mm - mem_map->map;
+		assert(idx < mem_map->count);
+
+		mm->size = va - mm->va;
+		m.va += mm->size + num_pages * SMALL_PAGE_SIZE;
+		m.pa += mm->size + num_pages * SMALL_PAGE_SIZE;
+		m.size -= mm->size + num_pages * SMALL_PAGE_SIZE;
+		grow_mem_map(mem_map);
+		ins_array_elem(mem_map->map, mem_map->count,
+			       sizeof(*mem_map->map), idx + 1, &m);
+	}
+}
+
 void core_mmu_unmap_pages(vaddr_t vstart, size_t num_pages)
 {
 	struct core_mmu_table_info tbl_info;
-	struct tee_mmap_region *mm;
 	size_t i;
 	unsigned int idx;
 	uint32_t exceptions;
 
 	exceptions = mmu_lock();
 
-	mm = find_map_by_va((void *)vstart);
-	if (!mm || !va_is_in_map(mm, vstart + num_pages * SMALL_PAGE_SIZE - 1))
-		panic("VA does not belong to any known mm region");
-
-	if (!core_mmu_is_dynamic_vaspace(mm) &&
-	    !mem_range_is_in_vcore_free(vstart, num_pages))
-		panic("Trying to unmap static region");
+	maybe_remove_from_mem_map(vstart, num_pages);
 
 	for (i = 0; i < num_pages; i++, vstart += SMALL_PAGE_SIZE) {
 		if (!core_mmu_find_table(NULL, vstart, UINT_MAX, &tbl_info))
@@ -2513,6 +2607,7 @@ static void *phys_to_virt_tee_ram(paddr_t pa, size_t len)
 		mmap = find_map_by_type_and_pa(MEM_AREA_TEE_RAM_RO, pa, len);
 	if (!mmap)
 		mmap = find_map_by_type_and_pa(MEM_AREA_TEE_RAM_RX, pa, len);
+
 	/*
 	 * Note that MEM_AREA_INIT_RAM_RO and MEM_AREA_INIT_RAM_RX are only
 	 * used with pager and not needed here.
@@ -2538,6 +2633,8 @@ void *phys_to_virt(paddr_t pa, enum teecore_memtypes m, size_t len)
 		va = phys_to_virt_tee_ram(pa, len);
 		break;
 	case MEM_AREA_SHM_VASPACE:
+	case MEM_AREA_NEX_DYN_VASPACE:
+	case MEM_AREA_TEE_DYN_VASPACE:
 		/* Find VA from PA in dynamic SHM is not yet supported */
 		va = NULL;
 		break;
@@ -2721,7 +2818,7 @@ void core_mmu_init_phys_mem(void)
 		/* Carve out test SDP memory */
 #ifdef TEE_SDP_TEST_MEM_BASE
 		if (TEE_SDP_TEST_MEM_SIZE) {
-			pa = vaddr_to_phys(TEE_SDP_TEST_MEM_BASE);
+			pa = TEE_SDP_TEST_MEM_BASE;
 			carve_out_core_mem(pa, pa + TEE_SDP_TEST_MEM_SIZE);
 		}
 #endif
